@@ -1,18 +1,18 @@
-import inspect
 import re
 import textwrap
 from ast import parse as ast_parse
-from pathlib import Path
-from typing import Optional, Union
+from copy import copy
+from typing import Optional
 
 import black
-import toml
 
-from snakefmt import DEFAULT_LINE_LENGTH
-from snakefmt.exceptions import InvalidParameterSyntax, InvalidPython, MalformattedToml
+import snakefmt.warnings as warnings
+from snakefmt.config import PathLike, read_black_config
+from snakefmt.exceptions import InvalidParameterSyntax, InvalidPython
 from snakefmt.parser.grammar import SnakeRule
 from snakefmt.parser.parser import Parser
 from snakefmt.parser.syntax import (
+    COMMENT_SPACING,
     TAB,
     Parameter,
     ParameterSyntax,
@@ -22,7 +22,6 @@ from snakefmt.parser.syntax import (
 )
 from snakefmt.types import TokenIterator
 
-PathLike = Union[Path, str]
 rule_like_formatted = {"rule", "checkpoint"}
 
 triple_quote_matcher = re.compile(
@@ -49,40 +48,13 @@ class Formatter(Parser):
         self.no_formatting_yet: bool = True
         self.last_recognised_keyword = ""
 
-        self.black_mode = black.FileMode(line_length=DEFAULT_LINE_LENGTH)
-        if black_config_file is not None:
-            self.read_black_config(black_config_file)
+        self.black_mode = read_black_config(black_config_file)
 
         if line_length is not None:
             self.black_mode.line_length = line_length
 
         super().__init__(snakefile)  # Call to parse snakefile
 
-    def read_black_config(self, path: PathLike) -> None:
-        if not Path(path).is_file():
-            raise FileNotFoundError(f"{path} is not a file.")
-
-        try:
-            pyproject_toml = toml.load(path)
-            config = pyproject_toml.get("tool", {}).get("black", {})
-        except toml.TomlDecodeError as error:
-            raise MalformattedToml(error)
-
-        valid_black_filemode_params = inspect.getfullargspec(black.FileMode).args
-
-        for key, val in config.items():
-            # this is due to FileMode param being string_normalise, but CLI being
-            # skip_string_normalise - https://github.com/snakemake/snakefmt/issues/73
-            if key.startswith("skip"):
-                key = key[5:]
-                val = not val
-
-            key = key.replace("-", "_")
-            if key not in valid_black_filemode_params:
-                continue
-
-            setattr(self.black_mode, key, val)
-
     def get_formatted(self) -> str:
         return self.result
 
@@ -166,18 +138,16 @@ class Formatter(Parser):
         self.last_recognised_keyword = param_context.keyword_name
 
     def run_black_format_str(self, string: str, target_indent: int) -> str:
-        # need to let black know about indentation as it affects line length
-        indent_line_length = target_indent * len(TAB)
-        saved_black_line_length = self.black_mode.line_length
-        contextual_line_length = max(0, saved_black_line_length - indent_line_length)
-        self.black_mode.line_length = contextual_line_length
+        # reduce black target line length according to how indented the code is
+        current_line_length = target_indent * len(TAB)
+        black_mode = copy(self.black_mode)
+        black_mode.line_length = max(0, black_mode.line_length - current_line_length)
         try:
-            fmted = black.format_str(string, mode=self.black_mode)
+            fmted = black.format_str(string, mode=black_mode)
         except black.InvalidInput as e:
             raise InvalidPython(
                 f"Got error:\n```\n{str(e)}\n```\n" f"while formatting code with black."
             ) from None
-        self.black_mode.line_length = saved_black_line_length
         return fmted
 
     def align_strings(self, string: str, target_indent: int) -> str:
@@ -213,9 +183,9 @@ class Formatter(Parser):
         inline_formatting: bool,
         single_param: bool = False,
     ) -> str:
+        string_indent = TAB * target_indent
         if inline_formatting:
             target_indent = 0
-        comments = f"\n{TAB * target_indent}".join(parameter.comments)
         val = str(parameter)
 
         try:
@@ -237,17 +207,24 @@ class Formatter(Parser):
             match_equal = re.match("(.*?) = (.*)", val, re.DOTALL)
             val = f"{match_equal.group(1)}={match_equal.group(2)}"
 
-        val = val.strip("\n")
-        if single_param:
-            result = f"{val}{comments}\n"
-        else:
-            result = f"{val},{comments}\n"
+        result = ""
+        if not inline_formatting:
+            for comment in parameter.pre_comments:
+                result += f"{string_indent}{comment}\n"
+        result += val.strip("\n")
+        if not single_param:
+            result += ","
+        post_comment_iter = iter(parameter.post_comments)
+        if parameter._has_inline_comment:
+            result += f"{COMMENT_SPACING}{next(post_comment_iter)}"
+        result += "\n"
+        for comment in post_comment_iter:
+            result += f"{string_indent}{comment}\n"
         return result
 
     def format_params(self, parameters: ParameterSyntax, in_rule: bool) -> str:
         target_indent = parameters.target_indent
         used_indent = TAB * (target_indent - 1)
-        result = f"{used_indent}{parameters.keyword_name}:{parameters.comment}"
 
         p_class = parameters.__class__
         single_param = issubclass(p_class, SingleParam)
@@ -256,15 +233,27 @@ class Formatter(Parser):
         if in_rule and p_class is not RuleInlineSingleParam:
             inline_fmting = False
 
+        result = f"{used_indent}{parameters.keyword_name}:"
         if inline_fmting:
             result += " "
+            prepended_comments = ""
+            if parameters.comment != "":
+                prepended_comments += f"{used_indent}{parameters.comment.lstrip()}\n"
+            param = next(iter(parameters.all_params))
+            for comment in param.pre_comments:
+                prepended_comments += f"{used_indent}{comment}\n"
+            if prepended_comments != "":
+                warnings.comment_relocation(parameters.keyword_name, param.line_nb)
+            result = f"{prepended_comments}{result}"
         else:
-            result += "\n"
-
-        for elem in parameters.all_params:
+            result += f"{parameters.comment}\n"
+        for param in parameters.all_params:
             result += self.format_param(
-                elem, target_indent, inline_fmting, single_param
+                param, target_indent, inline_fmting, single_param
             )
+        num_c = len(param.post_comments)
+        if num_c > 1 or (not param._has_inline_comment and num_c == 1):
+            warnings.block_comment_below(parameters.keyword_name, param.line_nb)
         return result
 
     def add_newlines(
