@@ -3,14 +3,14 @@ import re
 import sys
 from io import StringIO
 from pathlib import Path
-from typing import Dict, Iterable, Iterator, List, Optional, Pattern, Set, Union
+from typing import Iterator, List, Optional, Pattern, Set, Union
 
 import click
-import toml
-from black import find_project_root, get_gitignore
+from black import get_gitignore
 from pathspec import PathSpec
 
 from snakefmt import DEFAULT_LINE_LENGTH, __version__
+from snakefmt.config import inject_snakefmt_config
 from snakefmt.diff import Diff, ExitCode
 from snakefmt.formatter import Formatter
 from snakefmt.parser.parser import Snakefile
@@ -37,45 +37,6 @@ def construct_regex(regex: str) -> Pattern[str]:
     )
 
 
-def read_snakefmt_config(path: Optional[str]) -> Dict[str, str]:
-    """Parse Snakefmt configuration from provided toml."""
-    if path is None:
-        return dict()
-    try:
-        config_toml = toml.load(path)
-        config = config_toml.get("tool", {}).get("snakefmt", {})
-        config = {k.replace("--", "").replace("-", "_"): v for k, v in config.items()}
-        return config
-    except (toml.TomlDecodeError, OSError) as error:
-        raise click.FileError(
-            filename=path, hint=f"Error reading configuration file: {error}"
-        )
-
-
-def find_pyproject_toml(start_path: Iterable[str]) -> Optional[str]:
-    root = find_project_root(start_path)
-    config_file = root / "pyproject.toml"
-    return str(config_file) if config_file.is_file() else None
-
-
-def inject_snakefmt_config(
-    ctx: click.Context, param: click.Parameter, config_file: Optional[str] = None
-) -> Optional[str]:
-    """
-    If no config file argument provided, parses "pyproject.toml" if one exists.
-    Injects any parsed configuration into the relevant parameters to the click `ctx`.
-    """
-    if config_file is None:
-        config_file = find_pyproject_toml(ctx.params.get("src", ()))
-
-    config = read_snakefmt_config(config_file)
-
-    if ctx.default_map is None:
-        ctx.default_map = {}
-    ctx.default_map.update(config)  # type: ignore  # bad types in .pyi
-    return config_file
-
-
 def get_snakefiles_in_dir(
     path: Path, include: Pattern[str], exclude: Pattern[str], gitignore: PathSpec
 ) -> Iterator[Path]:
@@ -220,7 +181,7 @@ def main(
      `snakefmt - < Snakefile` to avoid this.
     """
     log_level = logging.DEBUG if verbose else logging.INFO
-    logging.basicConfig(format="[%(levelname)s] %(message)s", level=log_level)
+    toggle_logging(log_level)
 
     if not src:
         click.echo(
@@ -273,13 +234,15 @@ def main(
             logging.debug("Formatting from stdin")
             path = sys.stdin
         else:
-            logging.debug(f"Formatting {path}")
+            logging.debug("")
 
         try:
             original_content = path.read_text()
         except AttributeError:
             original_content = path.read()
 
+        if not path_is_stdin:
+            toggle_logging(log_level, path)
         try:
             snakefile = Snakefile(StringIO(original_content))
             formatter = Formatter(
@@ -288,7 +251,7 @@ def main(
             formatted_content = formatter.get_formatted()
         except Exception as error:
             if check:
-                logging.error(f"'{error.__class__.__name__}: {error}' in file {path}")
+                logging.error(f"'{error.__class__.__name__}: {error}'")
                 files_with_errors += 1
                 continue
             else:
@@ -302,6 +265,7 @@ def main(
             else:
                 files_unchanged += 1
 
+        toggle_logging(log_level)
         if diff or compact_diff:
             filename = "stdin" if path_is_stdin else str(path)
             click.echo(f"{'=' * 5}> Diff for {filename} <{'=' * 5}\n")
@@ -339,5 +303,17 @@ def main(
     logging.info("All done 🎉")
 
 
+log_template = "[%(levelname)s]{0} %(message)s"
+
+
+def toggle_logging(log_level, path: Optional[str] = None) -> None:
+    log_msg = log_template.format("")
+    if path is not None:
+        log_msg = log_template.format(f' In file "{path}": ')
+    logging.basicConfig(
+        format=log_msg, level=log_level, force=True,
+    )
+
+
 if __name__ == "__main__":
     main()
